{% extends 'hemlock/form-group.html' %}

{% block label %}
<div class="w-100">
    {% if q.error is not none %}
    <span {{ q.error_attrs.to_html() | safe }}>
        {{ q.error | safe}}
    </span>
    {% endif %}
    {% if q.label is not none %}
    {{ q.label | safe }}
    {% endif %}
</div>
{% endblock %}