/* Default css file */

/* Set body and html to viewport width */
body, html {
    height: 100%;
    width: 100%;
    background-color: #F5F5F5;
}
body {
    padding-top: 40px;
}

/* Form margins */
form.form-margins {
    margin: 20px 0px;
}

div.question {
    border-radius: 7px;
}

/* Question div formatting */
div.error {
    border-style: solid;
    border-width: thin;
    background-color: #f2dede;
    border-color: #ebcccc;
}

/* Choice formatting */
.choice {
    padding-left: 3px;
    padding-right: 3px;
    border-radius: 3px;
    background-clip: padding-box;
}
input:hover + label {
    background-color: lightgray;
}
input:checked + label {
    background-color: darkgray;
}
input[type="checkbox"]:checked:hover + label {
    background-color: #BEBEBE;
}