{% extends 'hemlock/input-group.html' %}

{% block input %}
<div class="custom-file">
    <input id="{{ q.key }}" name="{{ q.key }}" {{ q.input_attrs.to_html() | safe }}>
    <label for="{{ q.key }}" {{ q.file_label_attrs.to_html() | safe }}>
        {% if q.file_label is not none %}
        {{ q.file_label | safe }}
        {% endif %}
    </label>
</div>
{% endblock %}