{% set fg_class=q.form_group_class | join(" ") %}
<div id="{{ q.key }}-fg" class="{{ fg_class }}" {{ q.form_group_attrs.to_html() | safe }}>
    <div class="card-body">
        {% if q.error is not none %}
        <div {{ q.error_attrs.to_html() | safe }}>
            {{ q.error | safe}}
        </div>
        {% endif %}
        {% if q.label is not none %}
        {% block label %}
        <label class="w-100" for="{{ q.model_id }}">
            {{ q.label | safe }}
        </label>
        {% endblock %}
        {% endif %}
        {% block content %}{% endblock %}
        {% block form_text %}{% endblock %}
    </div>
</div>