{% extends 'hemlock/input-group.html' %}

{% block input %}
    {% set value = q.response if q.has_responded else q.default %}
    {% if value is none %}
        {% set value = "" %}
    {% endif %}
    <textarea id="{{ q.key }}" name="{{ q.key }}" {{ q.input_attrs.to_html() | safe }}>{{ value | safe}}</textarea>
{% endblock %}
{% block form_text %}
<small>Characters: <span id="{{ q.key }}-chars"></span></small>
<br/>
<small>Words: <span id="{{ q.key }}-words"></span></small>
{% endblock %}