"""Question polymorphs"""

from .blank import Blank
from .check import Binary, Check
from .choice import Choice, Option
from .dashboard import Dashboard
from .download import Download
from .file import File
from .input import Input
from .label import Label
from .range import Range, RangeInput
from .select import Select
from .textarea import Textarea